package com.emato.file.tunnel.config;

import com.emato.file.tunnel.config.properties.CPortProperties;
import com.emato.file.tunnel.config.properties.RabbitMQProperties;
import com.emato.file.tunnel.listener.ReportMessageListener;
import com.emato.file.tunnel.mq.callback.SimpleConfirmCallBack;
import com.emato.file.tunnel.mq.callback.SimpleReturnsCallBack;
import com.emato.file.tunnel.mq.converter.CustomMessageConverter;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.listener.MessageListenerContainer;
import org.springframework.amqp.rabbit.listener.RabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.amqp.RabbitProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;


/**
 * @author lhm
 * @version 1.0
 * 2021-08-24 16:04
 */
@Configuration
public class RabbitMQConfig {

    @Autowired
    private RabbitProperties rabbitProperties;

    @Autowired
    private RabbitMQProperties rabbitMQProperties;


    /**
     * 连接工厂
     * @return
     */
    @Bean
    public ConnectionFactory connectionFactory () {
        CachingConnectionFactory connectionFactory = new CachingConnectionFactory();
        connectionFactory.setUsername(rabbitProperties.getUsername());
        connectionFactory.setPassword(rabbitProperties.getPassword());
        connectionFactory.setHost(rabbitProperties.getHost());
        connectionFactory.setVirtualHost(rabbitProperties.getVirtualHost());
        connectionFactory.setPort(rabbitProperties.getPort());
        // 共用同一个Channel
        connectionFactory.setCacheMode(CachingConnectionFactory.CacheMode.CHANNEL);
        // 获取配置的Channel缓存大小
        connectionFactory.setChannelCacheSize(rabbitProperties.getCache().getChannel().getSize());
        // 消息到达broke后触发回调
        connectionFactory.setPublisherConfirmType(CachingConnectionFactory.ConfirmType.CORRELATED);
        return connectionFactory;
    }

    /**
     * SimpleRabbitListenerContainerFactory发现消息中有content_type有text就会默认将其转换成string类型
     * @return rabbit监听器容器工厂
     */
    @Bean
    public RabbitListenerContainerFactory<?> rabbitListenerContainerFactory(ConnectionFactory connectionFactory) {
        SimpleRabbitListenerContainerFactory containerFactory = new SimpleRabbitListenerContainerFactory();
        containerFactory.setConnectionFactory(connectionFactory);
        containerFactory.setMaxConcurrentConsumers(3);
        // 消息转换器
        containerFactory.setMessageConverter(new CustomMessageConverter());
        // 手动ack
        containerFactory.setAcknowledgeMode(AcknowledgeMode.MANUAL);
        return containerFactory;
    }

    @Bean
    public MessageListenerContainer messageListenerContainer (ConnectionFactory connectionFactory, CPortProperties cPortProperties, RabbitTemplate clearRabbitTemplate, RabbitMQProperties rabbitMQProperties) {
        RabbitListenerContainerFactory<?> rabbitListenerContainerFactory = rabbitListenerContainerFactory(connectionFactory);
        SimpleMessageListenerContainer listenerContainer = (SimpleMessageListenerContainer) rabbitListenerContainerFactory.createListenerContainer();

        // 支持621、623、625、311的上报
        listenerContainer.setQueueNames(
                rabbitMQProperties.getQ_normal_csp_to_tunnel_report_311(),
                rabbitMQProperties.getQ_normal_csp_to_tunnel_report_621(),

                rabbitMQProperties.getQ_normal_csp_to_tunnel_report_621()
        );
        listenerContainer.setupMessageListener(messageListener(cPortProperties, clearRabbitTemplate, rabbitMQProperties));

        return listenerContainer;
    }

    @Bean
    public MessageListener messageListener (CPortProperties cPortProperties, RabbitTemplate clearRabbitTemplate, RabbitMQProperties rabbitMQProperties) {
        return new ReportMessageListener(cPortProperties, clearRabbitTemplate, rabbitMQProperties);
    }

    @Bean
    public AmqpAdmin amqpAdmin (ConnectionFactory connectionFactory) {
        RabbitAdmin rabbitAdmin = new RabbitAdmin(connectionFactory);

        rabbitAdmin.declareExchange(reportDirectExchange());
        rabbitAdmin.declareExchange(receiptDirectExchange());

        rabbitAdmin.declareQueue(ceb621ReportQueue());
        rabbitAdmin.declareQueue(ceb311ReportQueue());

        rabbitAdmin.declareQueue(ceb621ReceiptQueue());
        rabbitAdmin.declareQueue(ceb311ReceiptQueue());
        rabbitAdmin.declareQueue(ceb623ReceiptQueue());
        rabbitAdmin.declareQueue(ceb625ReceiptQueue());
        rabbitAdmin.declareQueue(ceb816ReceiptQueue());

        rabbitAdmin.declareBinding(ceb621ReportBinding());
        rabbitAdmin.declareBinding(ceb311ReportBinding());

        rabbitAdmin.declareBinding(ceb621ReceiptBinding());
        rabbitAdmin.declareBinding(ceb311ReceiptBinding());
        rabbitAdmin.declareBinding(ceb623ReceiptBinding());
        rabbitAdmin.declareBinding(ceb625ReceiptBinding());
        rabbitAdmin.declareBinding(ceb816ReceiptBinding());

        return rabbitAdmin;
    }

    /** **************************************** 回执队列相关配置 ******************************************* */

    @Bean
    public DirectExchange receiptDirectExchange () {
        return new DirectExchange(rabbitMQProperties.getE_normal_tunnel_to_csp_receipt());
    }

    @Bean
    public RabbitTemplate receiptRabbitTemplate (ConnectionFactory connectionFactory) {
        RabbitTemplate rabbitTemplate = new RabbitTemplate(connectionFactory);
        rabbitTemplate.setExchange(rabbitMQProperties.getE_normal_tunnel_to_csp_receipt());
        rabbitTemplate.setConfirmCallback(new SimpleConfirmCallBack());
        rabbitTemplate.setReturnsCallback(new SimpleReturnsCallBack());
        return rabbitTemplate;
    }

    @Bean
    public Queue ceb621ReceiptQueue () {
        return new Queue(rabbitMQProperties.getQ_normal_tunnel_to_csp_receipt_621());
    }

    @Bean
    public Binding ceb621ReceiptBinding () {
        return BindingBuilder.bind(ceb621ReceiptQueue()).to(receiptDirectExchange()).with(rabbitMQProperties.getK_normal_tunnel_to_csp_receipt_621());
    }

    @Bean
    public Queue ceb311ReceiptQueue () {
        return new Queue(rabbitMQProperties.getQ_normal_tunnel_to_csp_receipt_311());
    }

    @Bean
    public Binding ceb311ReceiptBinding () {
        return BindingBuilder.bind(ceb311ReceiptQueue()).to(receiptDirectExchange()).with(rabbitMQProperties.getK_normal_tunnel_to_csp_receipt_311());
    }
    /* **************************************** 回执队列相关配置完毕 ******************************************* */


    /** **************************************** 上报报文相关配置 ******************************************* */

    @Bean
    public DirectExchange reportDirectExchange () {
        return new DirectExchange(rabbitMQProperties.getE_normal_csp_to_tunnel_report());
    }

    @Bean
    public RabbitTemplate reportRabbitTemplate (ConnectionFactory connectionFactory) {
        RabbitTemplate rabbitTemplate = new RabbitTemplate(connectionFactory);
        rabbitTemplate.setExchange(rabbitMQProperties.getE_normal_csp_to_tunnel_report());
        rabbitTemplate.setConfirmCallback(new SimpleConfirmCallBack());
        rabbitTemplate.setReturnsCallback(new SimpleReturnsCallBack());
        return rabbitTemplate;
    }

    @Bean
    public Queue ceb621ReportQueue () {
        return new Queue(rabbitMQProperties.getQ_normal_csp_to_tunnel_report_621());
    }

    @Bean
    public Binding ceb621ReportBinding () {
        return BindingBuilder.bind(ceb621ReportQueue()).to(reportDirectExchange()).with(rabbitMQProperties.getK_normal_csp_to_tunnel_report_621());
    }

    @Bean
    public Queue ceb311ReportQueue () {
        return new Queue(rabbitMQProperties.getQ_normal_csp_to_tunnel_report_311());
    }

    @Bean
    public Binding ceb311ReportBinding () {
        return BindingBuilder.bind(ceb621ReportQueue()).to(reportDirectExchange()).with(rabbitMQProperties.getK_normal_csp_to_tunnel_report_311());
    }

    /* **************************************** 上报报文相关配置完成 ******************************************* */


    /* **************************************** 清关时效性mq配置 ******************************************* */
    @Bean
    public RabbitTemplate clearRabbitTemplate (ConnectionFactory connectionFactory) {
        RabbitTemplate rabbitTemplate = new RabbitTemplate(connectionFactory);
        rabbitTemplate.setExchange(rabbitMQProperties.getE_normal_oms_order_to_handle_customs_clearance());
        rabbitTemplate.setRoutingKey(rabbitMQProperties.getK_normal_oms_order_to_handle_customs_clearance());
        return rabbitTemplate;
    }


    /* **************************************** 其他mq配置 ******************************************* */

    @Bean
    public Queue ceb623ReceiptQueue () {
        return new Queue(rabbitMQProperties.getQ_normal_tunnel_to_csp_receipt_623());
    }
    @Bean
    public Queue ceb623ReportQueue () {
        return new Queue(rabbitMQProperties.getQ_normal_csp_to_tunnel_report_623());
    }

    @Bean
    public Queue ceb625ReceiptQueue () {
        return new Queue(rabbitMQProperties.getQ_normal_tunnel_to_csp_receipt_625());
    }
    @Bean
    public Queue ceb625ReportQueue () {
        return new Queue(rabbitMQProperties.getQ_normal_csp_to_tunnel_report_625());
    }

    @Bean
    public Queue ceb816ReceiptQueue () {
        return new Queue(rabbitMQProperties.getQ_normal_tunnel_to_csp_receipt_816());
    }
    @Bean
    public Queue ceb816ReportQueue () {
        return new Queue(rabbitMQProperties.getQ_normal_csp_to_tunnel_report_816());
    }

    @Bean
    public Binding ceb623ReceiptBinding () {
        return BindingBuilder
                .bind(ceb623ReceiptQueue())
                .to(receiptDirectExchange())
                .with(rabbitMQProperties.getK_normal_tunnel_to_csp_receipt_623());
    }
    @Bean
    public Binding ceb623ReportBinding () {
        return BindingBuilder
                .bind(ceb623ReportQueue())
                .to(reportDirectExchange())
                .with(rabbitMQProperties.getK_normal_csp_to_tunnel_report_623());
    }

    @Bean
    public Binding ceb625ReceiptBinding () {
        return BindingBuilder
                .bind(ceb625ReceiptQueue())
                .to(receiptDirectExchange())
                .with(rabbitMQProperties.getK_normal_tunnel_to_csp_receipt_625());
    }
    @Bean
    public Binding ceb625ReportBinding () {
        return BindingBuilder
                .bind(ceb625ReportQueue())
                .to(reportDirectExchange())
                .with(rabbitMQProperties.getK_normal_csp_to_tunnel_report_625());
    }

    @Bean
    public Binding ceb816ReceiptBinding () {
        return BindingBuilder
                .bind(ceb816ReceiptQueue())
                .to(receiptDirectExchange())
                .with(rabbitMQProperties.getK_normal_tunnel_to_csp_receipt_816());
    }
    @Bean
    public Binding ceb816ReportBinding () {
        return BindingBuilder
                .bind(ceb816ReportQueue())
                .to(reportDirectExchange())
                .with(rabbitMQProperties.getK_normal_csp_to_tunnel_report_816());
    }

}
