package com.emato.file.tunnel.listener;

import cn.hutool.core.convert.ConvertException;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IORuntimeException;
import cn.hutool.core.util.XmlUtil;
import cn.hutool.json.JSONException;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.emato.file.tunnel.common.dto.CustomsClearanceHandleDto;
import com.emato.file.tunnel.common.response.Result;
import com.emato.file.tunnel.config.properties.CPortProperties;
import com.emato.file.tunnel.config.properties.RabbitMQProperties;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.listener.api.ChannelAwareMessageListener;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.handler.annotation.Header;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.*;

/**
 * @author lhm
 * @version 1.0
 * 2021-08-25 10:27
 */
@Slf4j
public class ReportMessageListener implements MessageListener, ChannelAwareMessageListener {

    @Autowired
    private CPortProperties cPortProperties;

    @Autowired
    private RabbitTemplate clearRabbitTemplate;

    @Autowired
    private RabbitMQProperties rabbitMQProperties;

    public ReportMessageListener(CPortProperties cPortProperties, RabbitTemplate clearRabbitTemplate, RabbitMQProperties rabbitMQProperties) {
        this.cPortProperties = cPortProperties;
        this.clearRabbitTemplate = clearRabbitTemplate;
        this.rabbitMQProperties = rabbitMQProperties;
    }

    /**
     * 处理上报的报文
     * @param message   报文数据(xml字符串)
     * @param channel   信道
     */
    public void process (String message, Channel channel, @Header(AmqpHeaders.DELIVERY_TAG) long tag) throws IOException {
        log.info("----- 监听报文上报, 收到消息, 开始上报! -----");
        try {
            JSONObject jsonObject = JSONUtil.parseObj(message);
            String msgFileName = jsonObject.get("msgFileName", String.class);
            String content = jsonObject.get("msgFileContent", String.class);
            String orderSn = jsonObject.get("orderSn", String.class);
            String path = cPortProperties.getPushDir()  + "\\" + msgFileName;
            // 去除空格和换行符
            content = XmlUtil.cleanComment(content);
            content = XmlUtil.cleanInvalid(content);
            content = content.trim();
            File file = FileUtil.writeString(content, path, StandardCharsets.UTF_8);
            channel.basicAck(tag, false);

            // 写入文件成功, 只有清单才发送, 发送清关时效性: 报文到达深关通目录时间
            if (cPortProperties.getCeb621()) {
                sendClearance(orderSn);
            }

            log.info("----- 监听报文上报, 文件: 【{}】 , 上报结束! -----", file.getName());
        } catch (JSONException e) {
            channel.basicAck(tag, false);
            log.error("----- 监听报文上报, json解析失败! 消息已丢弃! 消息体: {}, 异常: {} -----", message, e);
        } catch (IORuntimeException e) {
            requeue(channel, tag);
            log.error("----- 监听报文上报, 写入报文文件时出现异常! 消息已重新入队! 消息体: {}, 异常: {} -----", message, e);
        } catch (ConvertException e) {
            channel.basicAck(tag, false);
            log.error("----- 监听报文上报, json转换失败! 消息已丢弃! 消息体: {}, 异常: {} -----", message, e);
        } catch (Exception e) {
            channel.basicAck(tag, false);
            log.error("----- 监听报文上报, 上报出现未知异常! 消息已丢弃! 消息体: {}, 异常: {} -----", message, e);
        }
    }

    private void sendClearance (String orderSn) {
        try {
            CustomsClearanceHandleDto customsClearanceHandleDto = new CustomsClearanceHandleDto();
            customsClearanceHandleDto.setOrderSn(orderSn);
            customsClearanceHandleDto.setTimeType("cspSendTunnel");
            customsClearanceHandleDto.setTime(DateUtil.format(new Date(), DatePattern.NORM_DATETIME_FORMAT));
            List<CustomsClearanceHandleDto> customsClearanceHandleDtos = Collections.singletonList(customsClearanceHandleDto);
            Result result = Result.success(customsClearanceHandleDtos);
            clearRabbitTemplate.convertAndSend(JSONUtil.toJsonStr(result));
        } catch (Exception e) {
            log.error("----- 发送【报文写入到深关通目录时间】清关时效失败！ -----");
        }
    }

    private void requeue (Channel channel, long tag) {
        try {
            channel.basicNack(tag, false, true);
        } catch (IOException ioException) {
            log.error("----- 监听报文上报, 消息重新入队失败! -----", ioException);
            ioException.printStackTrace();
        }
    }

    /**
     * Callback for processing a received Rabbit message.
     * <p>Implementors are supposed to process the given Message,
     * typically sending reply messages through the given Session.
     *
     * @param message the received AMQP message (never <code>null</code>)
     * @param channel the underlying Rabbit Channel (never <code>null</code>)
     * @throws Exception Any.
     */
    @Override
    public void onMessage(Message message, Channel channel) throws Exception {
        process(new String(message.getBody(), StandardCharsets.UTF_8), channel, message.getMessageProperties().getDeliveryTag());
    }
}
